@@ -3,7 +3,7 @@
 """Module for fluid property integration.
 
 TESPy uses the CoolProp python interface for all fluid property functions. The
-tespy_fluid class allows the creation of lookup table for custom fluid
+TESPyFluid class allows the creation of lookup table for custom fluid
 mixtures.
 
 
@@ -36,7 +36,7 @@ from tespy.tools.helpers import single_fluid
 # %%
 
 
-class tespy_fluid:
+class TESPyFluid:
     r"""
     Allow the creation of lookup tables for specified mixtures.
 
@@ -46,7 +46,7 @@ class tespy_fluid:
     Parameters
     ----------
     alias : str
-        Alias for the fluid. Please note: The alias of a tespy_fluid class
+        Alias for the fluid. Please note: The alias of a TESPyFluid class
         object will always start with :code:`TESPy::`. See the example for more
         information!
 
@@ -81,15 +81,13 @@ class tespy_fluid:
     and temperature limits. We define dry air component wise as our custom
     fluid.
 
-    >>> from tespy.connections import connection
-    >>> from tespy.components.basics import sink, source
-    >>> from tespy.tools.fluid_properties import (tespy_fluid, h_mix_pT,
+    >>> from tespy.tools.fluid_properties import (TESPyFluid, h_mix_pT,
     ... s_mix_pT, v_mix_pT, visc_mix_pT)
-    >>> from tespy.networks.networks import network
+    >>> from tespy.networks import Network
     >>> import shutil
     >>> fluidvec = {'N2': 0.7552, 'O2': 0.2314, 'CO2': 0.0005, 'Ar': 0.0129}
     >>> p_arr = np.array([0.1, 10]) * 1e5
-    >>> myfluid = tespy_fluid('dry air', fluid=fluidvec, p_range=p_arr,
+    >>> myfluid = TESPyFluid('dry air', fluid=fluidvec, p_range=p_arr,
     ... T_range=[300, 1200])
 
     Check if the fluid creation was successful and compare some fluid
@@ -99,10 +97,10 @@ class tespy_fluid:
     viscosity. Specific volume and viscosity are absolute values, thus no
     difference is calculated.
 
-    >>> memorise.add_fluids({'air': 'HEOS'})
+    >>> Memorise.add_fluids({'air': 'HEOS'})
 
     >>> type(myfluid)
-    <class 'tespy.tools.fluid_properties.tespy_fluid'>
+    <class 'tespy.tools.fluid_properties.TESPyFluid'>
     >>> p = 3e5
     >>> fluid_props = [0, p, 0, {myfluid.alias: 1}]
     >>> T1 = 400
@@ -130,10 +128,10 @@ class tespy_fluid:
 
     The fluid had been saved automatically, load it now.
 
-    >>> loadfluid = tespy_fluid('dry air', fluid=fluidvec, p_range=p_arr,
+    >>> loadfluid = TESPyFluid('dry air', fluid=fluidvec, p_range=p_arr,
     ... path='./LUT')
     >>> type(loadfluid)
-    <class 'tespy.tools.fluid_properties.tespy_fluid'>
+    <class 'tespy.tools.fluid_properties.TESPyFluid'>
     >>> shutil.rmtree('./LUT', ignore_errors=True)
     """
 
@@ -158,7 +156,7 @@ class tespy_fluid:
         self.path = path
 
         # create look up tables
-        tespy_fluid.fluids[self.alias] = self
+        TESPyFluid.fluids[self.alias] = self
 
         self.fluids_back_ends = OrderedDict()
 
@@ -166,23 +164,24 @@ class tespy_fluid:
             try:
                 data = fluid.split('::')
                 back_end = data[0]
+                fluid = data[1]
             except IndexError:
                 back_end = 'HEOS'
             if fluid != self.alias:
                 self.fluids_back_ends[fluid] = back_end
 
-        memorise.add_fluids(self.fluids_back_ends)
+        Memorise.add_fluids(self.fluids_back_ends)
 
         # set up grid
         self.p = np.geomspace(p_range[0], p_range[1], 100)
 
         if T_range is None:
-            T_range = [max([memorise.value_range[f][2] for f
+            T_range = [max([Memorise.value_range[f][2] for f
                             in self.fluids_back_ends.keys()]) + 1, 2000]
 
         self.T = np.geomspace(T_range[0], T_range[1], 100)
 
-        memorise.add_fluids({self.alias: 'TESPy'})
+        Memorise.add_fluids({self.alias: 'TESPy'})
 
         params = {}
         params['h_pT'] = h_mix_pT
@@ -292,12 +291,12 @@ class tespy_fluid:
 
 
 # create dict for tespy fluids
-tespy_fluid.fluids = {}
+TESPyFluid.fluids = {}
 
 # %%
 
 
-class memorise:
+class Memorise:
     r"""Memorization of fluid properties."""
 
     @staticmethod
@@ -315,13 +314,13 @@ class memorise:
             Dict of fluid and corresponding CoolProp back end for fluid
             property memorization.
 
-        memorise_fluid_properties : bool
+        memorise_fluid_properties : boolean
             Activate or deactivate fluid property value memorisation. Default
             state is activated (:code:`True`).
 
         Note
         ----
-        The memorise class creates globally accessible variables for different
+        The Memorise class creates globally accessible variables for different
         fluid property calls as dictionaries:
 
         - T(p,h)
@@ -330,7 +329,7 @@ class memorise:
         - visc(p,h)
         - s(p,h)
 
-        Each dictionary uses the list of fluids passed to the memorise class as
+        Each dictionary uses the list of fluids passed to the Memorise class as
         identifier for the fluid property memorisation. The fluid properties
         are stored as numpy array, where each column represents the mass
         fraction of the respective fluid and the additional columns are the
@@ -348,31 +347,26 @@ class memorise:
         """
         # number of fluids
         num_fl = len(fluids)
-        if memorise_fluid_properties is True and num_fl > 0:
+        if memorise_fluid_properties and num_fl > 0:
             fl = tuple(fluids.keys())
             # fluid property tables
-            memorise.T_ph[fl] = np.empty((0, num_fl + 4), float)
-            memorise.T_ps[fl] = np.empty((0, num_fl + 5), float)
-            memorise.v_ph[fl] = np.empty((0, num_fl + 4), float)
-            memorise.visc_ph[fl] = np.empty((0, num_fl + 4), float)
-            memorise.s_ph[fl] = np.empty((0, num_fl + 4), float)
+            Memorise.T_ph[fl] = np.empty((0, num_fl + 4), float)
+            Memorise.T_ps[fl] = np.empty((0, num_fl + 5), float)
+            Memorise.v_ph[fl] = np.empty((0, num_fl + 4), float)
+            Memorise.visc_ph[fl] = np.empty((0, num_fl + 4), float)
+            Memorise.s_ph[fl] = np.empty((0, num_fl + 4), float)
 
-            msg = 'Added fluids ' + str(fl) + ' to memorise lookup tables.'
+            msg = (
+                'Added fluids ' + ', '.join(fl) +
+                ' to memorise lookup tables.')
             logging.debug(msg)
 
         for f, back_end in fluids.items():
-            if back_end == 'IF97':
-                msg = (
-                    'Due to a bug in the IF97 CoolProp back end, it is not '
-                    'possible to use this back end at the moment. For more '
-                    'information see '
-                    'https://github.com/CoolProp/CoolProp/issues/1918.')
-                logging.error(msg)
-                raise ValueError(msg)
-            elif f not in memorise.state.keys() and back_end != 'TESPy':
+            if f not in Memorise.state.keys() and back_end != 'TESPy':
                 # create CoolProp.AbstractState object
                 try:
-                    memorise.state[f] = CP.AbstractState(back_end, f)
+                    Memorise.state[f] = CP.AbstractState(back_end, f)
+                    Memorise.back_end[f] = back_end
                 except ValueError:
                     msg = (
                         'Could not find the fluid "' + f + '" in the fluid '
@@ -389,8 +383,8 @@ class memorise:
                 logging.debug(msg)
                 # pressure range
                 try:
-                    pmin = memorise.state[f].trivial_keyed_output(CP.iP_min)
-                    pmax = memorise.state[f].trivial_keyed_output(CP.iP_max)
+                    pmin = Memorise.state[f].trivial_keyed_output(CP.iP_min)
+                    pmax = Memorise.state[f].trivial_keyed_output(CP.iP_max)
                 except ValueError:
                     pmin = 1e4
                     pmax = 1e8
@@ -400,15 +394,15 @@ class memorise:
                     logging.warning(msg)
 
                 # temperature range
-                Tmin = memorise.state[f].trivial_keyed_output(CP.iT_min)
-                Tmax = memorise.state[f].trivial_keyed_output(CP.iT_max)
+                Tmin = Memorise.state[f].trivial_keyed_output(CP.iT_min)
+                Tmax = Memorise.state[f].trivial_keyed_output(CP.iT_max)
 
                 # value range for fluid properties
-                memorise.value_range[f] = [pmin, pmax, Tmin, Tmax]
+                Memorise.value_range[f] = [pmin, pmax, Tmin, Tmax]
 
                 try:
-                    molar_masses[f] = memorise.state[f].molar_mass()
-                    gas_constants[f] = memorise.state[f].gas_constant()
+                    molar_masses[f] = Memorise.state[f].molar_mass()
+                    gas_constants[f] = Memorise.state[f].gas_constant()
                 except ValueError:
                     try:
                         molar_masses[f] = CPPSI('M', f)
@@ -427,17 +421,17 @@ class memorise:
                 logging.debug(msg)
 
             elif back_end == 'TESPy':
-                pmin = tespy_fluid.fluids[f].p[0]
-                pmax = tespy_fluid.fluids[f].p[-1]
-                Tmin = tespy_fluid.fluids[f].T[0]
-                Tmax = tespy_fluid.fluids[f].T[-1]
+                pmin = TESPyFluid.fluids[f].p[0]
+                pmax = TESPyFluid.fluids[f].p[-1]
+                Tmin = TESPyFluid.fluids[f].T[0]
+                Tmax = TESPyFluid.fluids[f].T[-1]
                 msg = (
                     'Loading fluid property ranges for TESPy-fluid ' + f + '.')
                 logging.debug(msg)
                 # value range for fluid properties
-                memorise.value_range[f] = [pmin, pmax, Tmin, Tmax]
+                Memorise.value_range[f] = [pmin, pmax, Tmin, Tmax]
                 molar_masses[f] = 1 / molar_mass_flow(
-                    tespy_fluid.fluids[f].fluid)
+                    TESPyFluid.fluids[f].fluid)
                 gas_constants[f] = (
                     gas_constants['uni'] / molar_masses[f])
                 msg = ('Specifying fluid property ranges for pressure and '
@@ -458,41 +452,42 @@ class memorise:
         threshold = 3
         try:
             # delete memory
-            memorise.s_ph[fl] = memorise.s_ph[fl][
-                memorise.s_ph[fl][:, -1] > threshold]
-            memorise.s_ph[fl][:, -1] = 0
+            Memorise.s_ph[fl] = Memorise.s_ph[fl][
+                Memorise.s_ph[fl][:, -1] > threshold]
+            Memorise.s_ph[fl][:, -1] = 0
 
-            memorise.T_ph[fl] = memorise.T_ph[fl][
-                memorise.T_ph[fl][:, -1] > threshold]
-            memorise.T_ph[fl][:, -1] = 0
+            Memorise.T_ph[fl] = Memorise.T_ph[fl][
+                Memorise.T_ph[fl][:, -1] > threshold]
+            Memorise.T_ph[fl][:, -1] = 0
 
-            memorise.T_ps[fl] = memorise.T_ps[fl][
-                memorise.T_ps[fl][:, -1] > threshold]
-            memorise.T_ps[fl][:, -1] = 0
+            Memorise.T_ps[fl] = Memorise.T_ps[fl][
+                Memorise.T_ps[fl][:, -1] > threshold]
+            Memorise.T_ps[fl][:, -1] = 0
 
-            memorise.v_ph[fl] = memorise.v_ph[fl][
-                memorise.v_ph[fl][:, -1] > threshold]
-            memorise.v_ph[fl][:, -1] = 0
+            Memorise.v_ph[fl] = Memorise.v_ph[fl][
+                Memorise.v_ph[fl][:, -1] > threshold]
+            Memorise.v_ph[fl][:, -1] = 0
 
-            memorise.visc_ph[fl] = memorise.visc_ph[fl][
-                memorise.visc_ph[fl][:, -1] > threshold]
-            memorise.visc_ph[fl][:, -1] = 0
+            Memorise.visc_ph[fl] = Memorise.visc_ph[fl][
+                Memorise.visc_ph[fl][:, -1] > threshold]
+            Memorise.visc_ph[fl][:, -1] = 0
 
             msg = ('Dropping not frequently used fluid property values from '
-                   'memorise class for fluids ' + str(fl) + '.')
+                   'memorise class for fluids ' + ', '.join(fl) + '.')
             logging.debug(msg)
         except KeyError:
             pass
 
 
 # create memorise dictionaries
-memorise.state = {}
-memorise.T_ph = {}
-memorise.T_ps = {}
-memorise.v_ph = {}
-memorise.visc_ph = {}
-memorise.s_ph = {}
-memorise.value_range = {}
+Memorise.state = {}
+Memorise.back_end = {}
+Memorise.T_ph = {}
+Memorise.T_ps = {}
+Memorise.v_ph = {}
+Memorise.visc_ph = {}
+Memorise.s_ph = {}
+Memorise.value_range = {}
 
 # %%
 
@@ -530,23 +525,23 @@ def T_mix_ph(flow, T0=300):
     """
     # check if fluid properties have been calculated before
     fl = tuple(flow[3].keys())
-    memorisation = fl in memorise.T_ph.keys()
-    if memorisation is True:
-        a = memorise.T_ph[fl][:, :-2]
+    memorisation = fl in Memorise.T_ph.keys()
+    if memorisation:
+        a = Memorise.T_ph[fl][:, :-2]
         b = np.array([flow[1], flow[2]] + list(flow[3].values()))
         ix = np.where(np.all(abs(a - b) <= err, axis=1))[0]
 
         if ix.size == 1:
             # known fluid properties
-            memorise.T_ph[fl][ix, -1] += 1
-            return memorise.T_ph[fl][ix, -2][0]
+            Memorise.T_ph[fl][ix, -1] += 1
+            return Memorise.T_ph[fl][ix, -2][0]
 
     # unknown fluid properties
     fluid = single_fluid(flow[3])
     if fluid is None:
         # calculate the fluid properties for fluid mixtures
         valmin = max(
-            [memorise.value_range[f][2] for f in fl if flow[3][f] > err]
+            [Memorise.value_range[f][2] for f in fl if flow[3][f] > err]
         ) + 0.1
         if T0 < valmin or np.isnan(T0):
             T0 = valmin * 1.1
@@ -557,11 +552,11 @@ def T_mix_ph(flow, T0=300):
         # calculate fluid property for pure fluids
         val = T_ph(flow[1], flow[2], fluid)
 
-    if memorisation is True:
+    if memorisation:
         # memorise the newly calculated value
         new = np.asarray(
             [[flow[1], flow[2]] + list(flow[3].values()) + [val, 0]])
-        memorise.T_ph[fl] = np.append(memorise.T_ph[fl], new, axis=0)
+        Memorise.T_ph[fl] = np.append(Memorise.T_ph[fl], new, axis=0)
 
     return val
 
@@ -586,12 +581,14 @@ def T_ph(p, h, fluid):
     T : float
         Temperature T / K.
     """
-    if fluid in tespy_fluid.fluids.keys():
-        db = tespy_fluid.fluids[fluid].funcs['h_pT']
+    if fluid in TESPyFluid.fluids.keys():
+        db = TESPyFluid.fluids[fluid].funcs['h_pT']
         return newton(reverse_2d, reverse_2d_deriv, [db, p, h], 0)
+    elif Memorise.back_end[fluid] == 'IF97':
+        return entropy_iteration_IF97(p, h, fluid, 'T')
     else:
-        memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
-        return memorise.state[fluid].T()
+        Memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
+        return Memorise.state[fluid].T()
 
 
 def dT_mix_dph(flow, T0=300):
@@ -687,7 +684,7 @@ def dT_mix_ph_dfluid(flow, T0=300):
         else:
             vec_deriv += [0]
 
-    return np.asarray(vec_deriv)
+    return vec_deriv
 
 # %%
 
@@ -729,22 +726,22 @@ def T_mix_ps(flow, s, T0=300):
     """
     # check if fluid properties have been calculated before
     fl = tuple(flow[3].keys())
-    memorisation = fl in memorise.T_ps.keys()
-    if memorisation is True:
-        a = memorise.T_ps[fl][:, :-2]
+    memorisation = fl in Memorise.T_ps.keys()
+    if memorisation:
+        a = Memorise.T_ps[fl][:, :-2]
         b = np.asarray([flow[1], flow[2]] + list(flow[3].values()) + [s])
         ix = np.where(np.all(abs(a - b) <= err, axis=1))[0]
         if ix.size == 1:
             # known fluid properties
-            memorise.T_ps[fl][ix, -1] += 1
-            return memorise.T_ps[fl][ix, -2][0]
+            Memorise.T_ps[fl][ix, -1] += 1
+            return Memorise.T_ps[fl][ix, -2][0]
 
     # unknown fluid properties
     fluid = single_fluid(flow[3])
     if fluid is None:
         # calculate the fluid properties for fluid mixtures
         valmin = max(
-            [memorise.value_range[f][2] for f in fl if flow[3][f] > err]
+            [Memorise.value_range[f][2] for f in fl if flow[3][f] > err]
         ) + 0.1
         if T0 < valmin or np.isnan(T0):
             T0 = valmin * 1.1
@@ -756,11 +753,11 @@ def T_mix_ps(flow, s, T0=300):
         # calculate fluid property for pure fluids
         val = T_ps(flow[1], s, fluid)
 
-    if memorisation is True:
+    if memorisation:
         new = np.asarray(
             [[flow[1], flow[2]] + list(flow[3].values()) + [s, val, 0]])
         # memorise the newly calculated value
-        memorise.T_ps[fl] = np.append(memorise.T_ps[fl], new, axis=0)
+        Memorise.T_ps[fl] = np.append(Memorise.T_ps[fl], new, axis=0)
 
     return val
 
@@ -785,17 +782,17 @@ def T_ps(p, s, fluid):
     T : float
        Temperature T / K.
     """
-    if fluid in tespy_fluid.fluids.keys():
-        db = tespy_fluid.fluids[fluid].funcs['s_pT']
+    if fluid in TESPyFluid.fluids.keys():
+        db = TESPyFluid.fluids[fluid].funcs['s_pT']
         return newton(reverse_2d, reverse_2d_deriv, [db, p, s], 0)
     else:
-        memorise.state[fluid].update(CP.PSmass_INPUTS, p, s)
-        return memorise.state[fluid].T()
+        Memorise.state[fluid].update(CP.PSmass_INPUTS, p, s)
+        return Memorise.state[fluid].T()
 
 # %%
 
 
-def h_mix_pT(flow, T):
+def h_mix_pT(flow, T, force_gas=False):
     r"""
     Calculate the enthalpy from pressure and Temperature.
 
@@ -829,12 +826,12 @@ def h_mix_pT(flow, T):
     for fluid, x in flow[3].items():
         if x > err:
             ni = x / molar_masses[fluid]
-            h += h_pT(flow[1] * ni / n, T, fluid) * x
+            h += h_pT(flow[1] * ni / n, T, fluid, force_gas) * x
 
     return h
 
 
-def h_pT(p, T, fluid):
+def h_pT(p, T, fluid, force_gas=False):
     r"""
     Calculate the enthalpy from pressure and temperature for a pure fluid.
 
@@ -854,11 +851,19 @@ def h_pT(p, T, fluid):
     h : float
         Specific enthalpy h / (J/kg).
     """
-    if fluid in tespy_fluid.fluids.keys():
-        return tespy_fluid.fluids[fluid].funcs['h_pT'].ev(p, T)
+    if fluid in TESPyFluid.fluids.keys():
+        return TESPyFluid.fluids[fluid].funcs['h_pT'].ev(p, T)
     else:
-        memorise.state[fluid].update(CP.PT_INPUTS, p, T)
-        return memorise.state[fluid].hmass()
+        if force_gas:
+            if T < Memorise.state[fluid].trivial_keyed_output(CP.iT_critical):
+                Memorise.state[fluid].update(CP.PT_INPUTS, p, T)
+                h = Memorise.state[fluid].hmass()
+                Memorise.state[fluid].update(CP.QT_INPUTS, 1, T)
+                h_sat = Memorise.state[fluid].hmass()
+                return max(h, h_sat)
+
+        Memorise.state[fluid].update(CP.PT_INPUTS, p, T)
+        return Memorise.state[fluid].hmass()
 
 
 def dh_mix_pdT(flow, T):
@@ -939,14 +944,69 @@ def h_ps(p, s, fluid):
     h : float
         Specific enthalpy h / (J/kg).
     """
-    if fluid in tespy_fluid.fluids.keys():
-        db = tespy_fluid.fluids[fluid].funcs['s_pT']
+    if fluid in TESPyFluid.fluids.keys():
+        db = TESPyFluid.fluids[fluid].funcs['s_pT']
         T = newton(reverse_2d, reverse_2d_deriv, [db, p, s], 0)
-        return tespy_fluid.fluids[fluid].funcs['h_pT'].ev(p, T)
+        return TESPyFluid.fluids[fluid].funcs['h_pT'].ev(p, T)
     else:
-        memorise.state[fluid].update(CP.PSmass_INPUTS, p, s)
-        return memorise.state[fluid].hmass()
+        Memorise.state[fluid].update(CP.PSmass_INPUTS, p, s)
+        return Memorise.state[fluid].hmass()
 
+
+def h_ps_IF97(params, s):
+    r"""
+    Calculate the enthalpy from pressure and entropy for IF97 backend.
+
+    Parameters
+    ----------
+    fluid : str
+        Fluid name.
+
+    p : float
+        Pressure p / Pa.
+
+    s : float
+        Specific entropy h / (J/(kgK)).
+
+    Returns
+    -------
+    h : float
+        Specific enthalpy h / (J/kg).
+    """
+    Memorise.state[params[0]].update(CP.PSmass_INPUTS, params[1], s)
+    return Memorise.state[params[0]].hmass()
+
+
+def dh_pds_IF97(params, s):
+    r"""
+    Calculate the derivative of enthalpy to entropy at constant pressure.
+
+    For pure fluids only, required for IF97 entropy iteration only.
+
+    Parameters
+    ----------
+    p : float
+        Pressure p / Pa.
+
+    s : float
+        Specific entropy h / (J/(kgK)).
+
+    fluid : str
+        Fluid name.
+
+    Returns
+    -------
+    dh : float
+        Derivative of specific enthalpy dh / ds / K.
+    """
+    d = 1e-2
+    Memorise.state[params[0]].update(CP.PSmass_INPUTS, params[1], s + d)
+    h_upper = Memorise.state[params[0]].hmass()
+
+    Memorise.state[params[0]].update(CP.PSmass_INPUTS, params[1], s - d)
+    h_lower = Memorise.state[params[0]].hmass()
+
+    return (h_upper - h_lower) / (2 * d)
 # %%
 
 
@@ -979,12 +1039,12 @@ def h_mix_pQ(flow, Q):
         raise ValueError(msg)
 
     try:
-        memorise.state[fluid].update(CP.PQ_INPUTS, flow[1], Q)
+        Memorise.state[fluid].update(CP.PQ_INPUTS, flow[1], Q)
     except ValueError:
-        pcrit = memorise.state[fluid].trivial_keyed_output(CP.iP_critical)
-        memorise.state[fluid].update(CP.PQ_INPUTS, pcrit * 0.99, Q)
+        pcrit = Memorise.state[fluid].trivial_keyed_output(CP.iP_critical)
+        Memorise.state[fluid].update(CP.PQ_INPUTS, pcrit * 0.99, Q)
 
-    return memorise.state[fluid].hmass()
+    return Memorise.state[fluid].hmass()
 
 
 def dh_mix_dpQ(flow, Q):
@@ -1046,12 +1106,12 @@ def T_bp_p(flow):
     This function works for pure fluids only!
     """
     fluid = single_fluid(flow[3])
-    pcrit = memorise.state[fluid].trivial_keyed_output(CP.iP_critical)
+    pcrit = Memorise.state[fluid].trivial_keyed_output(CP.iP_critical)
     if flow[1] > pcrit:
-        memorise.state[fluid].update(CP.PQ_INPUTS, pcrit * 0.99, 1)
+        Memorise.state[fluid].update(CP.PQ_INPUTS, pcrit * 0.99, 1)
     else:
-        memorise.state[fluid].update(CP.PQ_INPUTS, flow[1], 1)
-    return memorise.state[fluid].T()
+        Memorise.state[fluid].update(CP.PQ_INPUTS, flow[1], 1)
+    return Memorise.state[fluid].T()
 
 
 def dT_bp_dp(flow):
@@ -1118,15 +1178,15 @@ def v_mix_ph(flow, T0=300):
     """
     # check if fluid properties have been calculated before
     fl = tuple(flow[3].keys())
-    memorisation = fl in memorise.v_ph.keys()
-    if memorisation is True:
-        a = memorise.v_ph[fl][:, :-2]
+    memorisation = fl in Memorise.v_ph.keys()
+    if memorisation:
+        a = Memorise.v_ph[fl][:, :-2]
         b = np.asarray([flow[1], flow[2]] + list(flow[3].values()))
         ix = np.where(np.all(abs(a - b) <= err, axis=1))[0]
         if ix.size == 1:
             # known fluid properties
-            memorise.v_ph[fl][ix, -1] += 1
-            return memorise.v_ph[fl][ix, -2][0]
+            Memorise.v_ph[fl][ix, -1] += 1
+            return Memorise.v_ph[fl][ix, -2][0]
 
     # unknown fluid properties
     fluid = single_fluid(flow[3])
@@ -1137,11 +1197,11 @@ def v_mix_ph(flow, T0=300):
         # calculate fluid property for pure fluids
         val = 1 / d_ph(flow[1], flow[2], fluid)
 
-    if memorisation is True:
+    if memorisation:
         # memorise the newly calculated value
         new = np.asarray(
             [[flow[1], flow[2]] + list(flow[3].values()) + [val, 0]])
-        memorise.v_ph[fl] = np.append(memorise.v_ph[fl], new, axis=0)
+        Memorise.v_ph[fl] = np.append(Memorise.v_ph[fl], new, axis=0)
 
     return val
 
@@ -1166,13 +1226,15 @@ def d_ph(p, h, fluid):
     d : float
         Density d / (kg/:math:`\mathrm{m}^3`).
     """
-    if fluid in tespy_fluid.fluids.keys():
-        db = tespy_fluid.fluids[fluid].funcs['h_pT']
+    if fluid in TESPyFluid.fluids.keys():
+        db = TESPyFluid.fluids[fluid].funcs['h_pT']
         T = newton(reverse_2d, reverse_2d_deriv, [db, p, h], 0)
-        return tespy_fluid.fluids[fluid].funcs['d_pT'].ev(p, T)
+        return TESPyFluid.fluids[fluid].funcs['d_pT'].ev(p, T)
+    elif Memorise.back_end[fluid] == 'IF97':
+        return entropy_iteration_IF97(p, h, fluid, 'rho')
     else:
-        memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
-        return memorise.state[fluid].rhomass()
+        Memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
+        return Memorise.state[fluid].rhomass()
 
 # %%
 
@@ -1198,8 +1260,8 @@ def Q_ph(p, h, fluid):
         Vapor mass fraction.
     """
     try:
-        memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
-        return memorise.state[fluid].Q()
+        Memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
+        return Memorise.state[fluid].Q()
     except (KeyError, ValueError):
         return np.nan
 
@@ -1354,11 +1416,11 @@ def d_pT(p, T, fluid):
     d : float
         Density d / (kg/:math:`\mathrm{m}^3`).
     """
-    if fluid in tespy_fluid.fluids.keys():
-        return tespy_fluid.fluids[fluid].funcs['d_pT'].ev(p, T)
+    if fluid in TESPyFluid.fluids.keys():
+        return TESPyFluid.fluids[fluid].funcs['d_pT'].ev(p, T)
     else:
-        memorise.state[fluid].update(CP.PT_INPUTS, p, T)
-        return memorise.state[fluid].rhomass()
+        Memorise.state[fluid].update(CP.PT_INPUTS, p, T)
+        return Memorise.state[fluid].rhomass()
 
 # %%
 
@@ -1392,15 +1454,15 @@ def visc_mix_ph(flow, T0=300):
     """
     # check if fluid properties have been calculated before
     fl = tuple(flow[3].keys())
-    memorisation = fl in memorise.visc_ph.keys()
-    if memorisation is True:
-        a = memorise.visc_ph[fl][:, :-2]
+    memorisation = fl in Memorise.visc_ph.keys()
+    if memorisation:
+        a = Memorise.visc_ph[fl][:, :-2]
         b = np.asarray([flow[1], flow[2]] + list(flow[3].values()))
         ix = np.where(np.all(abs(a - b) <= err, axis=1))[0]
         if ix.size == 1:
             # known fluid properties
-            memorise.visc_ph[fl][ix, -1] += 1
-            return memorise.visc_ph[fl][ix, -2][0]
+            Memorise.visc_ph[fl][ix, -1] += 1
+            return Memorise.visc_ph[fl][ix, -2][0]
 
     # unknown fluid properties
     fluid = single_fluid(flow[3])
@@ -1411,11 +1473,11 @@ def visc_mix_ph(flow, T0=300):
         # calculate the fluid properties for pure fluids
         val = visc_ph(flow[1], flow[2], fluid)
 
-    if memorisation is True:
+    if memorisation:
         # memorise the newly calculated value
         new = np.asarray(
             [[flow[1], flow[2]] + list(flow[3].values()) + [val, 0]])
-        memorise.visc_ph[fl] = np.append(memorise.visc_ph[fl], new, axis=0)
+        Memorise.visc_ph[fl] = np.append(Memorise.visc_ph[fl], new, axis=0)
     return val
 
 
@@ -1439,13 +1501,15 @@ def visc_ph(p, h, fluid):
     visc : float
         Viscosity visc / Pa s.
     """
-    if fluid in tespy_fluid.fluids.keys():
-        db = tespy_fluid.fluids[fluid].funcs['h_pT']
+    if fluid in TESPyFluid.fluids.keys():
+        db = TESPyFluid.fluids[fluid].funcs['h_pT']
         T = newton(reverse_2d, reverse_2d_deriv, [db, p, h], 0)
-        return tespy_fluid.fluids[fluid].funcs['visc_pT'].ev(p, T)
+        return TESPyFluid.fluids[fluid].funcs['visc_pT'].ev(p, T)
+    elif Memorise.back_end[fluid] == 'IF97':
+        return entropy_iteration_IF97(p, h, fluid, 'visc')
     else:
-        memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
-        return memorise.state[fluid].viscosity()
+        Memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
+        return Memorise.state[fluid].viscosity()
 
 # %%
 
@@ -1516,11 +1580,11 @@ def visc_pT(p, T, fluid):
     visc : float
         Viscosity visc / Pa s.
     """
-    if fluid in tespy_fluid.fluids.keys():
-        return tespy_fluid.fluids[fluid].funcs['visc_pT'].ev(p, T)
+    if fluid in TESPyFluid.fluids.keys():
+        return TESPyFluid.fluids[fluid].funcs['visc_pT'].ev(p, T)
     else:
-        memorise.state[fluid].update(CP.PT_INPUTS, p, T)
-        return memorise.state[fluid].viscosity()
+        Memorise.state[fluid].update(CP.PT_INPUTS, p, T)
+        return Memorise.state[fluid].viscosity()
 
 # %%
 
@@ -1554,15 +1618,15 @@ def s_mix_ph(flow, T0=300):
     """
     # check if fluid properties have been calculated before
     fl = tuple(flow[3].keys())
-    memorisation = fl in memorise.s_ph.keys()
-    if memorisation is True:
-        a = memorise.s_ph[fl][:, :-2]
+    memorisation = fl in Memorise.s_ph.keys()
+    if memorisation:
+        a = Memorise.s_ph[fl][:, :-2]
         b = np.asarray([flow[1], flow[2]] + list(flow[3].values()))
         ix = np.where(np.all(abs(a - b) <= err, axis=1))[0]
         if ix.size == 1:
             # known fluid properties
-            memorise.s_ph[fl][ix, -1] += 1
-            return memorise.s_ph[fl][ix, -2][0]
+            Memorise.s_ph[fl][ix, -1] += 1
+            return Memorise.s_ph[fl][ix, -2][0]
 
     # unknown fluid properties
     fluid = single_fluid(flow[3])
@@ -1573,11 +1637,11 @@ def s_mix_ph(flow, T0=300):
         # calculate fluid property for pure fluids
         val = s_ph(flow[1], flow[2], fluid)
 
-    if memorisation is True:
+    if memorisation:
         # memorise the newly calculated value
         new = np.asarray(
             [[flow[1], flow[2]] + list(flow[3].values()) + [val, 0]])
-        memorise.s_ph[fl] = np.append(memorise.s_ph[fl], new, axis=0)
+        Memorise.s_ph[fl] = np.append(Memorise.s_ph[fl], new, axis=0)
 
     return val
 
@@ -1602,18 +1666,20 @@ def s_ph(p, h, fluid):
     s : float
         Specific entropy s / (J/(kgK)).
     """
-    if fluid in tespy_fluid.fluids.keys():
-        db = tespy_fluid.fluids[fluid].funcs['h_pT']
+    if fluid in TESPyFluid.fluids.keys():
+        db = TESPyFluid.fluids[fluid].funcs['h_pT']
         T = newton(reverse_2d, reverse_2d_deriv, [db, p, h], 0)
-        return tespy_fluid.fluids[fluid].funcs['s_pT'].ev(p, T)
+        return TESPyFluid.fluids[fluid].funcs['s_pT'].ev(p, T)
+    elif Memorise.back_end[fluid] == 'IF97':
+        return entropy_iteration_IF97(p, h, fluid, 's')
     else:
-        memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
-        return memorise.state[fluid].smass()
+        Memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
+        return Memorise.state[fluid].smass()
 
 # %%
 
 
-def s_mix_pT(flow, T):
+def s_mix_pT(flow, T, force_gas=False):
     r"""
     Calculate the entropy from pressure and temperature.
 
@@ -1648,15 +1714,15 @@ def s_mix_pT(flow, T):
     fluid_comps = {}
 
     tespy_fluids = [
-        s for s in flow[3].keys() if s in tespy_fluid.fluids.keys()]
+        s for s in flow[3].keys() if s in TESPyFluid.fluids.keys()]
     for f in tespy_fluids:
-        for it, x in tespy_fluid.fluids[f].fluid.items():
+        for it, x in TESPyFluid.fluids[f].fluid.items():
             if it in fluid_comps.keys():
                 fluid_comps[it] += x * flow[3][f]
             else:
                 fluid_comps[it] = x * flow[3][f]
 
-    fluids = [s for s in flow[3].keys() if s not in tespy_fluid.fluids.keys()]
+    fluids = [s for s in flow[3].keys() if s not in TESPyFluid.fluids.keys()]
     for f in fluids:
         if f in fluid_comps.keys():
             fluid_comps[f] += flow[3][f]
@@ -1669,14 +1735,14 @@ def s_mix_pT(flow, T):
     for fluid, x in fluid_comps.items():
         if x > err:
             pp = flow[1] * x / (molar_masses[fluid] * n)
-            s += s_pT(pp, T, fluid) * x
+            s += s_pT(pp, T, fluid, force_gas) * x
             s -= (x * gas_constants[fluid] / molar_masses[fluid] *
                   np.log(pp / flow[1]))
 
     return s
 
 
-def s_pT(p, T, fluid):
+def s_pT(p, T, fluid, force_gas):
     r"""
     Calculate the entropy from pressure and temperature for a pure fluid.
 
@@ -1696,11 +1762,19 @@ def s_pT(p, T, fluid):
     s : float
         Specific entropy s / (J/(kgK)).
     """
-    if fluid in tespy_fluid.fluids.keys():
-        return tespy_fluid.fluids[fluid].funcs['s_pT'].ev(p, T)
+    if fluid in TESPyFluid.fluids.keys():
+        return TESPyFluid.fluids[fluid].funcs['s_pT'].ev(p, T)
     else:
-        memorise.state[fluid].update(CP.PT_INPUTS, p, T)
-        return memorise.state[fluid].smass()
+        if force_gas:
+            if T < Memorise.state[fluid].trivial_keyed_output(CP.iT_critical):
+                Memorise.state[fluid].update(CP.PT_INPUTS, p, T)
+                s = Memorise.state[fluid].smass()
+                Memorise.state[fluid].update(CP.QT_INPUTS, 1, T)
+                s_sat = Memorise.state[fluid].smass()
+                return max(s, s_sat)
+
+        Memorise.state[fluid].update(CP.PT_INPUTS, p, T)
+        return Memorise.state[fluid].smass()
 
 
 def ds_mix_pdT(flow, T):
@@ -1729,3 +1803,153 @@ def ds_mix_pdT(flow, T):
     """
     d = 0.1
     return (s_mix_pT(flow, T + d) - s_mix_pT(flow, T - d)) / (2 * d)
+
+
+def isentropic(inflow, outflow, T0=300):
+    r"""
+    Calculate the enthalpy at the outlet after isentropic process.
+
+    Parameters
+    ----------
+    inflow : list
+        Inflow fluid property vector containing mass flow, pressure, enthalpy
+        and fluid composition.
+
+    outflow : list
+        Outflow fluid property vector containing mass flow, pressure, enthalpy
+        and fluid composition.
+
+    Returns
+    -------
+    h_s : float
+        Enthalpy after isentropic state change.
+
+        .. math::
+
+            h_\mathrm{s} = \begin{cases}
+            h\left(p_{out}, s\left(p_{in}, h_{in}\right) \right) &
+            \text{pure fluids}\\
+            h\left(p_{out}, s\left(p_{in}, T_{in}\right) \right) &
+            \text{mixtures}\\
+            \end{cases}
+    """
+    fluid = single_fluid(inflow[3])
+    if fluid is not None:
+        return h_ps(outflow[1], s_ph(inflow[1], inflow[2], fluid), fluid)
+    else:
+        s_mix = s_mix_ph(inflow)
+        return h_mix_ps(outflow, s_mix, T0=T0)
+
+
+def calc_physical_exergy(conn, p0, T0):
+    r"""
+    Calculate specific physical exergy.
+
+    Physical exergy is allocated to a thermal and a mechanical share according
+    to :cite:`Morosuk2005`.
+
+    Parameters
+    ----------
+    conn : tespy.connections.connection.Connection
+        Connection to calculate specific physical exergy for.
+
+    p0 : float
+        Ambient pressure p0 / Pa.
+
+    T0 : float
+        Ambient temperature T0 / K.
+
+    Returns
+    -------
+    e_ph : tuple
+        Specific thermal and mechanical exergy
+        (:math:`e^\mathrm{T}`, :math:`e^\mathrm{M}`) in J / kg.
+
+        .. math::
+
+            e^\mathrm{T} = \left( h - h \left( p, T_0 \right) \right) -
+            T_0 \cdot \left(s - s\left(p, T_0\right)\right)
+
+            e^\mathrm{M}=\left(h\left(p,T_0\right)-h\left(p_0,T_0\right)\right)
+            -T_0\cdot\left(s\left(p, T_0\right)-s\left(p_0,T_0\right)\right)
+
+            e^\mathrm{PH} = e^\mathrm{T} + e^\mathrm{M}
+    """
+    h_T0_p = h_mix_pT([0, conn.p.val_SI, 0, conn.fluid.val], T0)
+    s_T0_p = s_mix_pT([0, conn.p.val_SI, 0, conn.fluid.val], T0)
+    ex_therm = (conn.h.val_SI - h_T0_p) - T0 * (conn.s.val_SI - s_T0_p)
+    h0 = h_mix_pT([0, p0, 0, conn.fluid.val], T0)
+    s0 = s_mix_pT([0, p0, 0, conn.fluid.val], T0)
+    ex_mech = (h_T0_p - h0) - T0 * (s_T0_p - s0)
+    return ex_therm, ex_mech
+
+
+def entropy_iteration_IF97(p, h, fluid, output):
+    r"""
+    Generate the state of IF97::water via entropy iteration.
+
+    Parameters
+    ----------
+    p : float
+        Pressure p / Pa.
+
+    h : float
+        Specific enthalpy h / (J/kg).
+
+    fluid : str
+        Fluid name.
+
+    Returns
+    -------
+    T : float
+        Temperature T / K.
+    """
+    # region 1 exclusive issue!
+    Memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
+    if p <= 16.529164252605 * 1e6:
+        h_at_ph = Memorise.state[fluid].hmass()
+        deviation = abs(h_at_ph - h)
+        if deviation / h > 0.001:
+            # region 1, where isenthalpic lines are tangent to saturation dome
+            if p > 1e6 and p < 1e7 and h > 2700000 and h < 2850000:
+                smin = 5750
+                smax = 6500
+            # bottom left corner in Ts diagram
+            elif h < 10000:
+                smin = 0
+                smax = 50
+            else:
+                # proximity to saturated liquid
+                Memorise.state[fluid].update(CP.PQ_INPUTS, p, 0)
+                h_sat_l = Memorise.state[fluid].hmass()
+                if abs(h - h_sat_l) / h_sat_l < 1e-1:
+                    if p < 1000:
+                        smin = 0
+                    elif p < 60000:
+                        smin = Memorise.state[fluid].smass() * 0.9
+                    else:
+                        smin = Memorise.state[fluid].smass() * 0.95
+
+                    Memorise.state[fluid].update(CP.PQ_INPUTS, p, 0.3)
+                    smax = Memorise.state[fluid].smass()
+                # all others
+                else:
+                    Memorise.state[fluid].update(CP.HmassP_INPUTS, h, p)
+                    s0 = Memorise.state[fluid].smass()
+                    smin = 0.8 * s0
+                    smax = 1.2 * s0
+
+            s0 = (smax + smin) / 2
+            s = newton(func=h_ps_IF97, deriv=dh_pds_IF97, params=[fluid, p],
+                       y=h, val0=s0, valmin=smin, valmax=smax, max_iter=5,
+                       tol_rel=1e-3, tol_mode='rel')
+            Memorise.state[fluid].update(CP.PSmass_INPUTS, p, s)
+
+    if output == 'T':
+        return Memorise.state[fluid].T()
+    elif output == 's':
+        return Memorise.state[fluid].smass()
+    elif output == 'rho':
+        return Memorise.state[fluid].rhomass()
+    else:
+        return Memorise.state[fluid].viscosity()
